class PopupMiddleware(object):
    """
    This middleware must always be enabled when using yawd-elfinder.
    Place it **before** the :class:`django.middleware.common.CommonMiddleware`
    in your ``MIDLEWARE_CLASSESS`` setting.
    
    yawd-admin builds upon the original django admin application.
    Some admin widgets open pop-up windows where yawd-admin uses
    a modal window. Since original AdminModel views attempt to return
    pop-up window values to the parent through the ``opener`` javascript
    variable, the iframes used in yawd-elfinder will not work as expected.
    This middleare implements an easy fix, replacing ``opener`` with
    the ``parent`` variable, which is appropriate for iframes.
    """
    def process_response(self, request, response):
        """
        This method is called right after a view is processed and has
        returned an HttpResponse object.
        """
        
        #responses will not always have a content attribute starting
        #from Django 1.5
        if response.status_code == 200 and hasattr(response, 
            'content') and response.content.startswith(
                '<!DOCTYPE html><html><head><title></title></head><body>'
                '<script type="text/javascript">opener.dismissAddAnotherPopup(window,'):
            
            response.content = response.content.replace('<!DOCTYPE html><html><head><title></title></head><body>'
                '<script type="text/javascript">opener.dismissAddAnotherPopup(window,',
                '<!DOCTYPE html><html><head><title></title></head><body>'
                '<script>parent.dismissAddAnotherPopup(window,')
            
        return response



from django import forms
from itertools import chain
from django.contrib.contenttypes.models import ContentType
from django.utils.safestring import mark_safe

class ContentTypeSelect(forms.Select):
    def __init__(self, object_id,  attrs=None, choices=()):
        self.object_id = 'id_%s' % object_id
        super(ContentTypeSelect, self).__init__(attrs, choices)
        
    def render(self, name, value, attrs=None, choices=()):
        output = super(ContentTypeSelect, self).render(name, value, attrs, choices)
        
        choiceoutput = ' var %s_choice_urls = {' % (attrs['id'],)
        
        for ctype in ContentType.objects.filter(pk__in=[int(c[0])for c in chain(self.choices, choices) if c[0]]):
            choiceoutput += '    \'%s\' : \'../../../%s/%s?t=%s\','  % ( str(ctype.pk), 
                    ctype.app_label, ctype.model, ctype.model_class()._meta.pk.name)
        
        choiceoutput += '};'
        
        output += ('<script>'
                   '(function($) {'
                   '  $(document).ready( function() {'
                   '%(choiceoutput)s'
                   '    $(\'#%(id)s\').change(function (){'
                   '        $(\'#%(fk_id)s\').val(\'\');'
                   '        $(\'#lookup_%(fk_id)s\').attr(\'href\',%(id)s_choice_urls[$(this).val()]+"&pop=1").siblings(\'strong\').html(\'\');'
                   '    });'
                   '  });'
                   '})(yawdadmin.jQuery);'
                   '</script>' % { 'choiceoutput' : choiceoutput, 
                                    'id' : attrs['id'],
                                    'fk_id' : self.object_id
                                  })
        return mark_safe(u''.join(output))


from oauth2client.client import flow_from_clientsecrets
from django.conf import settings

ADMIN_GOOGLE_ANALYTICS = {
    'client_secrets' : '',
    'token_file_name' : '',
    'profile_id' : '',
    'admin_root_url' : 'http://localhost:8000/admin/',
    'interval' : 30 #how many days back should we look for data?
}

try:
    ADMIN_GOOGLE_ANALYTICS.update(settings.ADMIN_GOOGLE_ANALYTICS)
except AttributeError:
    pass

if ADMIN_GOOGLE_ANALYTICS['client_secrets'] and ADMIN_GOOGLE_ANALYTICS['profile_id'] and ADMIN_GOOGLE_ANALYTICS['token_file_name']:
    ADMIN_GOOGLE_ANALYTICS_FLOW = flow_from_clientsecrets( ADMIN_GOOGLE_ANALYTICS['client_secrets'],
        scope='https://www.googleapis.com/auth/analytics.readonly', redirect_uri='%soauth2callback/' % (
        ADMIN_GOOGLE_ANALYTICS['admin_root_url']))
    #always prompt for authentication
    ADMIN_GOOGLE_ANALYTICS_FLOW.params['approval_prompt'] = 'force'
else:
     ADMIN_GOOGLE_ANALYTICS_FLOW = None
    


try:
    import json
except:
    import simplejson as json
from oauth2client import xsrfutil
from oauth2client.file import Storage
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseBadRequest, HttpResponseRedirect, Http404
from django.utils.translation import ugettext as _
from django.views.generic import TemplateView, View
from conf import settings as ls

class AppOptionView(TemplateView):
    template_name = 'admin/options.html'
    
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('yawdadmin.change_appoption'):
            raise PermissionDenied
        return super(AppOptionView, self).dispatch(request, *args, **kwargs)
    
    def get_form_kwargs(self):
        """
        Returns the keyword arguments for instantiating the 
        form. Copied form the generic FormView class-based view
        """
        kwargs = {}
        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data':self.request.POST, 
                'files':self.request.FILES, 
                })
        return kwargs

    def get_context_data(self, **kwargs):
        from yawdadmin import admin_site
        context = super(AppOptionView, self).get_context_data(**kwargs)
        context['optionset_admin'] = admin_site.get_optionset_admin(self.kwargs['optionset_label'])(**self.get_form_kwargs())
        context['title'] = '%s %s' % (context['optionset_admin'].verbose_name, _('Configuration'))
        return context
    
    def post(self, request, *args, **kwargs):
        """
        Validate the form and save the options upon success
        """
        context = self.get_context_data()
        if context['optionset_admin'].form.is_valid():
            context['optionset_admin'].save()
            messages.add_message(self.request, messages.SUCCESS, _('The options were succesfully saved.'))
        return self.render_to_response(context)
    
    def put(self, request, *args, **kwargs):
        return self.post(request, *args, **kwargs)
    
class AnalyticsAuthView(View):
    """
    This view implements the oauth2 authentication callback.
    It stores the user credential to a file and redirects the user
    to the admin index page on success. 
    """
    permanent = False
    
    def get(self, request, *args, **kwargs):
        #check view
        valid_analytics_view(request)
        
        if not ('state' in request.REQUEST and xsrfutil.validate_token(settings.SECRET_KEY, request.REQUEST['state'], request.user)):
            return  HttpResponseBadRequest()
        
        credential = ls.ADMIN_GOOGLE_ANALYTICS_FLOW.step2_exchange(request.REQUEST) #@UndefinedVariable
        storage = Storage(ls.ADMIN_GOOGLE_ANALYTICS['token_file_name'])
        storage.put(credential)
        
        messages.add_message(self.request, messages.SUCCESS, _('The user was successfully connected.'))
        return HttpResponseRedirect(reverse('admin:analytics'))
    
class AnalyticsConfigView(TemplateView):
    """
    Admin view for the google analytics functionality. The view is 
    accessible through the top bar navigation.
    """
    template_name = 'admin/analytics.html'
    
    def get_context_data(self, **kwargs):
    
        #check view
        valid_analytics_view(self.request)
        #get original context data
        context = super(AnalyticsConfigView, self).get_context_data(**kwargs)
        #load the token file
        try:
            dat_file = open(ls.ADMIN_GOOGLE_ANALYTICS['token_file_name'], 'r')
            analytics = json.loads(dat_file.read())
            dat_file.close()
        except (IOError, json.decoder.JSONDecodeError):
            analytics = {}
        
        context['analytics_info'] = {
            'profile' : ls.ADMIN_GOOGLE_ANALYTICS['profile_id'],
            'interval' : ls.ADMIN_GOOGLE_ANALYTICS['interval'],
            'data' : analytics
        }
        
        return context
    
class AnalyticsConnectView(View):
    """
    Connect a new user to the Google Analytics API
    """
    def get(self, request, *args, **kwargs):
        
        #check view
        valid_analytics_view(request)
        try: 
            #Empty the token file 
            dat_file = open(ls.ADMIN_GOOGLE_ANALYTICS['token_file_name'], 'w+')
            dat_file.write('')
            dat_file.close()
        except:
            messages.add_message(self.request, messages.ERROR, _('The server does not have permissions to write to the token file. Please contact your system administrator.'))
            return HttpResponseRedirect(reverse('admin:analytics'))

        #Initialize flow
        ls.ADMIN_GOOGLE_ANALYTICS_FLOW.params['state'] = xsrfutil.generate_token(settings.SECRET_KEY, request.user) #@UndefinedVariable
        return HttpResponseRedirect(ls.ADMIN_GOOGLE_ANALYTICS_FLOW.step1_get_authorize_url()) #@UndefinedVariable
    
def valid_analytics_view(request):
    """
    Check if the user is superuser and analytics functionality is enabled. 
    """
    if not request.user.is_superuser:
            raise PermissionDenied
    
    if not ls.ADMIN_GOOGLE_ANALYTICS_FLOW:
        raise Http404


import re
from django.db import models
from django.core.exceptions import ValidationError

class OptionNameField(models.CharField):
    
    def clean(self, value):
        value = super(OptionNameField, self).clean(value)
        if not re.match(r'[a-zA-Z_]+', value):
            raise ValidationError('Only letters and underscores are allowed.')
        return value


import datetime, time
from oauth2client.client import AccessTokenRefreshError
from django.core.cache import cache
from django.utils import simplejson as json
from django.utils.translation import get_language
from django.utils.encoding import smart_str
from yawdadmin import admin_site
from conf import settings as ls
from models import AppOption

def get_option(optionset_label, name, current_only=True):
    """
    Return the value of an option.
    """
    try:
        option = AppOption.objects.get(optionset_label=optionset_label, name=name)
    except AppOption.DoesNotExist:
        return None
    
    optionset_admin = admin_site.get_optionset_admin(optionset_label)
    return get_option_value(optionset_admin, option, current_only)

def get_options(optionset_label, current_only=True):
    """
    Return all options for this app_label as dictionary with the option name
    being the key. 
    """
    try:
        options = AppOption.objects.filter(optionset_label=optionset_label)
    except:
        return {}
    
    optionset_admin = admin_site.get_optionset_admin(optionset_label)
    
    option_dict = {}
    for option in options:
        option_dict[smart_str(option.name)] = get_option_value(optionset_admin, option, current_only)
        
    return option_dict

def get_option_value(optionset_admin, db_option, current_only):
    """
    Given an AppOption object, return its value for the current language.
    """
    
    name = smart_str(db_option.name)
    if not name in optionset_admin.options:
        return None
    
    field = optionset_admin.options[name]

    if not db_option.lang_dependant:
        return field.to_python(db_option.value) if db_option.value else '' 

    value_dict = {}
    for key, value in json.loads(db_option.value).items():
        value_dict[smart_str(key)] = value

    if current_only:
        curr_lang = get_language()
        if curr_lang in value_dict:
            return field.to_python(value_dict[curr_lang]) if value_dict[curr_lang] else ''
    else:
        for key in value_dict:
            value_dict[key] = field.to_python(value_dict[key])
            return value_dict
        
def get_analytics_data(http):
    
    #try to get cached data
    data = cache.get('yawdadmin_ga', None)
    if data:
        return data
    
    from apiclient.discovery import build
    from apiclient.errors import HttpError
            
    service = build('analytics', 'v3', http=http)
    end_date = datetime.datetime.now()
    start_date = end_date + datetime.timedelta(-ls.ADMIN_GOOGLE_ANALYTICS['interval'])
    
    try:
        pie_data = service.data().ga().get(ids = 'ga:' + ls.ADMIN_GOOGLE_ANALYTICS['profile_id'],
            start_date = start_date.strftime('%Y-%m-%d'), end_date = end_date.strftime('%Y-%m-%d'),
            metrics='ga:visits', dimensions='ga:date,ga:visitorType').execute()

        summed_data = service.data().ga().get(ids = 'ga:' + ls.ADMIN_GOOGLE_ANALYTICS['profile_id'],
            start_date = start_date.strftime('%Y-%m-%d'), end_date = end_date.strftime('%Y-%m-%d'),
            metrics='ga:pageviews, ga:visitors, ga:avgTimeOnSite, ga:entranceBounceRate, ga:percentNewVisits').execute()
    except HttpError as e:
        return { 'error' : e.resp.reason }
    except AccessTokenRefreshError:
        return { 'error' : 'refresh' }
    
    data = {
        'summed' : {
            'visits' : pie_data['totalsForAllResults']['ga:visits'],
            'pageviews' : summed_data['rows'][0][0],
            'visitors' : summed_data['rows'][0][1],
            'avg_time' : time.strftime('%H:%M:%S', time.gmtime(float(summed_data['rows'][0][2]))),
            'bounce_rate' : round(float(summed_data['rows'][0][3]), 2),
            'new_visits' : round(float(summed_data['rows'][0][4]), 2),
        },
        'chart' : _extract_chart_data(pie_data, start_date, end_date)
    }
    
    cache.set('yawdadmin_ga', data, 3600)
    return data
    
def _extract_chart_data(pie_data, start_date, end_date):
    """
    Format the Google Analytics data for use within an Area Chart.
    """
    
    def update_record(record):
        visits = 0
        for key in ('new','returning'):
            if not key in record:
                record[key] = 0
            visits += record[key]
        record['total'] = visits
        
    date = start_date
    current_row = date.strftime('%Y%m%d')
    data = [{ 'date' : date.strftime('%A, %B %d, %Y')}]
    
    #calculate chart-ready data
    for row in pie_data['rows']:
        while row[0] != current_row:
            update_record(data[len(data)-1])
            date = date + datetime.timedelta(1)
            data.append({'date' : date.strftime('%A, %B %d, %Y')})
            current_row = date.strftime('%Y%m%d')

        if row[1] == 'New Visitor':
            data[len(data)-1]['new'] = int(row[2])
        else:
            data[len(data)-1]['returning'] = int(row[2])
    
    update_record(data[len(data)-1])
    
    return data


import httplib2
from functools import update_wrapper
from oauth2client import xsrfutil
from oauth2client.file import Storage
from django.conf.urls import patterns, url
from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse, NoReverseMatch
from django.http import HttpResponseRedirect
from django.utils.text import capfirst
from django.views.decorators.cache import never_cache
from conf import settings as ls
from models import AppOption
from views import AppOptionView, AnalyticsAuthView, AnalyticsConfigView, AnalyticsConnectView

_optionset_labels = {}

class YawdAdminSite(AdminSite):
    
    def __init__(self, *args, **kwargs):
        super(YawdAdminSite, self).__init__(*args, **kwargs)
        self._top_menu = {}
        
    def check_dependencies(self):
        """
        Override the default method to check that the 
        :class:`yawdadmin.middleware.PopupMiddleware` is installed
        and ``yawdadmin`` is found in the INSTALLED_APPS setting **before**
        ``django.contrib.admin``.
        """
        super(YawdAdminSite, self).check_dependencies()
        if not AppOption._meta.installed:  #@UndefinedVariable
            raise ImproperlyConfigured("Put 'yawdadmin' in your "
                "INSTALLED_APPS setting in order to use the yawd-admin application.")
        if settings.INSTALLED_APPS.index('yawdadmin') > settings.INSTALLED_APPS.index('django.contrib.admin'):
            raise ImproperlyConfigured("Put 'yawdadmin' before 'django.contrib.admin' "
                "in your INSTALLED_APPS setting to use the yawd-admin application") 
        if not 'yawdadmin.middleware.PopupMiddleware' in settings.MIDDLEWARE_CLASSES:
            raise ImproperlyConfigured("Put 'yawdadmin.middleware.PopupMiddleware' "
                "in your MIDDLEWARE_CLASSES setting in order to use the yawd-admin application.")
        
    def get_urls(self):
        global _optionset_labels
        
        def wrap(view, cacheable=False):
            def wrapper(*args, **kwargs):
                return self.admin_view(view, cacheable)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        urlpatterns = patterns('',
            url(r'^configuration-options/(?P<optionset_label>%s)/$' % '|'.join(_optionset_labels.keys()), wrap(AppOptionView.as_view()), name='optionset-label-options'),
            url(r'^oauth2callback/$', wrap(AnalyticsAuthView.as_view()), name='oauth2-callback'),
            url(r'^google-analytics/$', wrap(AnalyticsConfigView.as_view()), name='analytics'),
            url(r'^google-analytics/connect/$', wrap(AnalyticsConnectView.as_view()), name='analytics-connect'))

        urlpatterns += super(YawdAdminSite, self).get_urls() 
        return urlpatterns
        
    def register_top_menu_item(self, item, icon_class=''):
        app_labels = []
        for model, model_admin in self._registry.iteritems():
            if not model._meta.app_label in app_labels:
                app_labels.append(model._meta.app_label)
        
        if isinstance(item, basestring) and item in app_labels:
            if not item in self._top_menu:
                self._top_menu[item] = icon_class
        else:
            raise Exception("Item has to be a valid app_label")
        
    def unregister_top_menu_item(self, item):
        if isinstance(item, basestring) and item in self._top_menu:
            del self._top_menu[item]
        else:
            raise Exception("Item is not registered in the top menu")
    
    def top_menu(self, request):
        user = request.user
        app_dict = {}

        for model, model_admin in self._registry.items():
            app_label = model._meta.app_label
            if app_label in self._top_menu:
                has_module_perms = user.has_module_perms(app_label)

                if has_module_perms:
                    perms = model_admin.get_model_perms(request)
        
                    # Check whether user has any perm for this module.
                    # If so, add the module to the model_list.
                    if True in perms.values():
                        info = (app_label, model._meta.module_name)
                        model_dict = {
                            'name': capfirst(model._meta.verbose_name_plural),
                            'perms': perms,
                        }
                        if perms.get('change', False):
                            try:
                                model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=self.name)
                            except NoReverseMatch:
                                pass
                        if perms.get('add', False):
                            try:
                                model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=self.name)
                            except NoReverseMatch:
                                pass

                        model_dict['order'] = model_admin.order if hasattr(model_admin, 'order') else 3
                        model_dict['separator'] = model_admin.separator if hasattr(model_admin, 'separator') else False

                        if app_label in app_dict:
                            app_dict[app_label]['models'].append(model_dict)
                        else:
                            app_dict[app_label] = {
                                'name': app_label.title(),
                                'icon': self._top_menu[app_label],
                                'app_url': reverse('admin:app_list', kwargs={'app_label': app_label}, current_app=self.name),
                                'has_module_perms': has_module_perms,
                                'models': [model_dict],
                            }
        app_list = app_dict.values()
        app_list.sort(key=lambda x: x['name'])

        # Sort the models alphabetically within each app.
        for app in app_list:
            app['models'].sort(key=lambda x: x['order'])

        return app_list
    
    def register_options(self, optionset_admin):
        """
        Allows an application to register admin options like so::

            admin_site.register_options(OptionSetAdminClass)
        """
        global _optionset_labels
        if not optionset_admin.optionset_label in _optionset_labels:
            #Add admin optionset to the registry
            _optionset_labels[optionset_admin.optionset_label] = optionset_admin
            #Initialize options
            optionset_admin()
    
    def unregister_options(self, optionset_admin):
        optionset_label = optionset_admin.optionset_label
        if optionset_label in _optionset_labels:
            AppOption.objects.filter(optionset_label=optionset_label).delete()
            del _optionset_labels[optionset_label]
        
    def get_option_admin_urls(self):
        """
        Return a list of key-value pairs, containing all available optionset urls 
        """
        global _optionset_labels

        if not hasattr(self, 'option_admin_urls'):
            option_urls = []
            for option in _optionset_labels:
                option_urls.append({ 'optionset_label' : _optionset_labels[option].verbose_name, 'url' : reverse('admin:optionset-label-options', kwargs={ 'optionset_label' : option }, current_app='admin')})
            self.option_admin_urls = option_urls

        return self.option_admin_urls
    
    def get_optionset_admin(self, optionset_label):
        """
        Returns the OptionSetAdmin class for this label
        """
        global _optionset_labels
        if optionset_label in _optionset_labels:
            return _optionset_labels[optionset_label]
        
    @never_cache
    def index(self, request, extra_context={}):
        """
        This index view implementation adds Google Analytics
        integration so that you can view important metrics 
        right from the administrator interface.
        """
        #if admin google analytics is enabled
        if ls.ADMIN_GOOGLE_ANALYTICS_FLOW:
            #get the analytics user credentials
            storage = Storage(ls.ADMIN_GOOGLE_ANALYTICS['token_file_name'])
            credential = storage.get()
            
            if credential is None or credential.invalid == True:
                extra_context['ga_data'] = { 'error' : 'authentication' }
            else:
                #get the data
                from utils import get_analytics_data
                http = httplib2.Http()
                extra_context['ga_data'] = get_analytics_data(credential.authorize(http))
            
        return super(YawdAdminSite, self).index(request, extra_context)


from django.db import models
from fields import OptionNameField

class AppOption(models.Model):
    optionset_label = models.CharField(max_length=50)
    name = OptionNameField(max_length=50)
    value = models.TextField(null=True)
    lang_dependant = models.BooleanField(default=False)
    
    class Meta:
        unique_together = ('optionset_label', 'name')
        ordering = ['optionset_label', 'lang_dependant']

    def __unicode__(self):
        return u'%s.%s' % (self.optionset_label, self.name)


import re
from django import template
from django.conf import settings
from django.core import urlresolvers
from yawdadmin import admin_site
from yawdadmin.conf import settings as ls

register = template.Library()

@register.inclusion_tag('admin/includes/topmenu.html', takes_context=True)
def admin_top_menu(context):        
    return {
        'perms' : context['perms'],
        'top_menu' : admin_site.top_menu(context['request']),
        'homeurl' : urlresolvers.reverse('admin:index'),
        'user' : context['user'],
        'langs' : context['langs'] if 'langs' in context else [],
        'default_lang': context['default_lang'] if 'default_lang' in context else None,
        'clean_url' : context['clean_url'] if 'clean_url' in context else '',
        'LANGUAGE_CODE' : context['LANGUAGE_CODE'],
        'optionset_labels' : admin_site.get_option_admin_urls(),
        'analytics' : context['user'].is_superuser and ls.ADMIN_GOOGLE_ANALYTICS_FLOW,
    }

@register.simple_tag
def clean_media(media):
    media._js = [i for i in media._js if not re.match(
        r'%sadmin/js/((jquery(\.init)?|collapse|admin/RelatedObjectLookups)(\.min)?\.)js' % settings.STATIC_URL, i)]
    return media



import re, copy
from django import forms
from django.utils import simplejson as json
from utils import get_options
from models import AppOption

#decouple yawd-admin and yawd-translations applications
try:
    from translations.utils import get_supported_languages
except:
    #mimic the yawd-translations get_supported languages() behavior
    #for the languages defined in settings.LANGUAGES
    from django.conf import settings
    def get_supported_languages():
        return [x[0] for x in settings.LANGUAGES]
    
_optionsetadmin_classes = {}

class SiteOption(object):
    order_counter = 0
    
    def __init__(self, field=None, lang_dependant=False):
        
        if not isinstance(field, forms.Field):
            raise Exception('The field must be a valid field instance')
        
        self.field = field
        self.lang_dependant = lang_dependant
        #remember the order in which fields were initialized
        self.order_counter = SiteOption.order_counter
        SiteOption.order_counter += 1

class OptionSetBase(type):
    """
    metaclass for all OptionSets
    """ 
    def __new__(self, name, bases, attrs):
        super_new = super(OptionSetBase, self).__new__
        parents = [b for b in bases if isinstance(b, OptionSetBase)]
        if not parents:
            # If this isn't a subclass of Model, don't do anything special.
            return super_new(self, name, bases, attrs)
        
        # Create the class.
        # Add the options and form fields
        module = attrs.pop('__module__')
        
        try:
            optionset_label = attrs.pop('optionset_label')
        except KeyError:
            optionset_label = None

        if not optionset_label or not re.match(r'[a-zA-z-]+', optionset_label):
            raise TypeError("optionset_label must be set and contain only letters and underscores")
        
        # Because of the way imports happen (recursively), it may or may not be
        # the first time this model tries to register with the framework. 
        # There should only be one class for each OptionSetAdmin.
        global _optionsetadmin_classes
        if optionset_label in _optionsetadmin_classes:
            return _optionsetadmin_classes[optionset_label]

        try:        
            verbose_name = attrs.pop('verbose_name')
        except KeyError:
            verbose_name = optionset_label.title().replace ('-', ' ')
        
        new_class = super_new(self, name, bases, {
            '__module__': module, 
            'options' : {},
            'lang_options' : {},
            'attrs' : [],
            'optionset_label' : optionset_label,
            'verbose_name' : verbose_name,
        })
        
        for attr, value in attrs.items():
            if not isinstance(value, SiteOption):
                raise TypeError('Invalid attribute %s - should be a SiteOption instance' % attr)
            
            if not hasattr(value.field, 'label') or not value.field.label:
                value.field.label = attr.title().replace('_', ' ')

            new_class.attrs.append((attr, value))
            new_class.options[attr] = value.field 
            
        new_class.attrs.sort(lambda (attr1, value1), (attr2, value2): cmp(value1.order_counter, value2.order_counter))
        
        _optionsetadmin_classes[optionset_label] = new_class
        return new_class

def _init_option(optionset_label, name, siteoption):
    db_option, created = AppOption.objects.get_or_create(name = name, optionset_label = optionset_label)
    
    if siteoption.lang_dependant:
        ret = {}
        for l in get_supported_languages():
            ret[l] = ''
        db_option.value = json.dumps(ret)
    else:
        ret = ''
        db_option.value = ret
    
    if db_option.lang_dependant != siteoption.lang_dependant:
        db_option.lang_dependant = siteoption.lang_dependant
            
    if created:
        db_option.save()
    
    return ret
    
class OptionSetAdmin(object):
    __metaclass__ = OptionSetBase
    
    def __init__(self, **kwargs):
        self.form = forms.Form(**kwargs)

        #load option values from the database
        self.value_dict = get_options(optionset_label = self.optionset_label, current_only=False)
        self.formfields = []
        self.langformfields = {}

        for (attr, value) in self.attrs:

            if not attr in self.value_dict:
                self.value_dict[attr] = _init_option(self.optionset_label, attr, value)

            if value.lang_dependant:
                for lang in get_supported_languages():
                    #generate the form field
                    field_name = '%s_%s' % (attr, lang)
                    lang_field = copy.deepcopy(value.field)
                    lang_field.label = '%s (%s)' % (lang_field.label, lang.upper())
                    
                    self.form.fields[field_name] = lang_field
                    try:
                        self.form.fields[field_name].initial = self.value_dict[attr][lang]
                    except KeyError:
                        self.form.fields[field_name].initial = ''
                    
                    #add to land dependant options
                    self.lang_options[field_name] = (attr, lang)
                    
                    #langformfields fieldset
                    if not lang in self.langformfields:
                        self.langformfields[lang] = []
                    self.langformfields[lang].append(self.form[field_name])
            else:
                self.form.fields[attr] = value.field
                self.form.fields[attr].initial = self.value_dict[attr]
                self.formfields.append(self.form[attr])
    
    def save(self):
        #this dictionary is used to collect lang-dependant field values
        lang_dependant_value_dict = {}

        for field, value in self.form.cleaned_data.iteritems():
            #prepare value for save
            prep_value = self.form.fields[field].prepare_value(value)
             
            if field in self.lang_options:
                original_field = self.lang_options[field][0]
                lang = self.lang_options[field][1]
                
                if not original_field in lang_dependant_value_dict:
                    lang_dependant_value_dict[original_field] = {}
                
                lang_dependant_value_dict[original_field].update({
                    lang : prep_value
                })
            else:
                if self.value_dict[field] != prep_value:
                    AppOption.objects.filter(optionset_label=self.optionset_label, name=field).update(value = prep_value)
    
        #save lang-dependant options
        for key, value in lang_dependant_value_dict.iteritems():
            if self.value_dict[key] != value:
                AppOption.objects.filter(optionset_label=self.optionset_label, name=key).update(value = json.dumps(value))


# -*- coding: utf-8 -*-
from django import template

register = template.Library()

@register.filter
def divide(value, num):
    return int(value / num)

@register.filter
def app_title(value):
    return value.replace('_',' ')

@register.filter
def utfupper(value):
    orig = [u'Ά', u'Έ', u'Ή', u'Ί', u'Ό', u'Ύ', u'Ώ']
    rep = [u'Α', u'Ε', u'Η', u'Ι', u'Ο', u'Υ', u'Ω']
    return u''.join([rep[orig.index(x)] if x in orig else x for x in value.upper()])
        


#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
      name='yawd-admin',
      url='http://yawd.eu/open-source-projects/yawd-admin/',
      version = '0.5.0',
      description='An administration website for Django',
      author='yawd',
      author_email='info@yawd.eu',
      packages=find_packages(),
      license='BSD',
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries'
        ],
      include_package_data = True,
      install_requires = [
        "django >= 1.4",
        "oauth2client"
        ],
      zip_safe=False
)